package nl.innovalor.experiment.testproject.events;

import nl.innovalor.experiment.testproject.events.transaction_before_event.exception.TransactionBeforeExceptionEvent;

import javax.annotation.Resource;
import javax.ejb.EJBTransactionRolledbackException;
import javax.ejb.Singleton;
import javax.enterprise.inject.spi.CDI;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.Status;
import javax.transaction.TransactionSynchronizationRegistry;

@Singleton
public class PersistenceBean {

    @PersistenceContext
    EntityManager entityManager;

    @Resource
    TransactionSynchronizationRegistry tsr;

    /**
     * starts transaction for the event entity, and fires the event
     *
     * @param event to start the transaction & fire
     */
    public void startTransactionAndFireEventSpecific(TransactionBeforeExceptionEvent event) {
        try {
            event.genId();
            entityManager.persist(event);
            CDI.current().getBeanManager().fireEvent(event);
        } catch (EJBTransactionRolledbackException e) {
            if (tsr.getTransactionStatus() != Status.STATUS_MARKED_ROLLBACK) {
                throw new IllegalStateException("The transaction should be rolled back");
            }
        } catch (EventException e) {
            if (tsr.getTransactionStatus() == Status.STATUS_ACTIVE) {
                throw new IllegalStateException("This should be a rolled back exception, and the transaction should be rolled back");
            }
        }
    }
}
